﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.BusinessLogicsContracts;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopDataModels.Models;
using Microsoft.Extensions.Logging;
using Microsoft.VisualBasic.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace BlacksmithWorkshop
{
	public partial class FormShop : Form
	{
		private readonly ILogger _logger;
		private readonly IShopLogic _logic;
		public int? _id;
		private Dictionary<int, (IManufactureModel, int)> _manufactures;
		public FormShop(ILogger<FormShop> logger, IShopLogic logic)
		{
			InitializeComponent();
			_logger = logger;
			_logic = logic;
			_manufactures = new();
		}
		private void FormShop_Load(object sender, EventArgs e)
		{
			if (_id.HasValue)
			{
				_logger.LogInformation("Загрузка магазина");
				try
				{
					var shop = _logic.ReadElement(new ShopSearchModel { Id = _id });
					if (shop != null)
					{
						textBoxName.Text = shop.ShopName;
						textBoxAddress.Text = shop.Address;
						dateTimePicker.Text = shop.OpeningDate.ToString();
						_manufactures = shop.ShopManufactures;
					}
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка загрузки магазина");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				}
			}
		}
		private void LoadData()
		{
			_logger.LogInformation("Загрузка товаров магазина");
			try
			{
				if (_manufactures != null)
				{
					foreach (var manufactures in _manufactures)
					{
						dataGridView.Rows.Add(new object[] { manufactures.Key, manufactures.Value.Item1.ManufactureName,
						manufactures.Value.Item1.Price, manufactures.Value.Item2 });
					}
				}
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки изделий магазина");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
				MessageBoxIcon.Error);
			}
		}
		private void CancelButton_Click(object sender, EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
			Close();
		}
		private void SaveButton_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(textBoxName.Text))
			{
				MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (string.IsNullOrEmpty(textBoxAddress.Text))
			{
				MessageBox.Show("Заполните адрес", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			_logger.LogInformation("Сохранение магазина");
			try
			{
				var model = new ShopBindingModel
				{
					Id = _id ?? 0,
					ShopName = textBoxName.Text,
					Address = textBoxAddress.Text,
					OpeningDate = dateTimePicker.Value.Date,
					ShopManufactures = _manufactures
				};
				var operationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);
				if (!operationResult)
				{
					throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
				}
				MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
				DialogResult = DialogResult.OK;
				Close();
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка сохранения магазина");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
	}
}
